import reti.*;
import java.io.*;
import java.net.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.border.*;

public class Client extends JFrame implements ActionListener
{

	public static void main(String[] args)
	{
		JFrame client = new Client();
		client.show();
	}

	public Client()
	{
		setTitle("IMAGE RETRIEVAL BY CONTENT");
		setSize(550,430);
		setResizable(false);
		addWindowListener( new WindowAdapter()
			{
				public void windowClosing(WindowEvent e)
				{ System.exit(0); }
			});
		Border etched = BorderFactory.createEtchedBorder();
		Border titled1 = BorderFactory.createTitledBorder(etched,"QUERY");
		JPanel p1 = new JPanel();
		p1.setBorder(titled1);		
		try
		{
			image = new JComboBox();
			image.setEditable(false);
			ObjectInputStream inputquery = new ObjectInputStream(new FileInputStream("query.dat"));
			listaquery = new ImageFeature[20];
			for(int i=0; i<20; i++) 
			{
				ImageFeature aux = (ImageFeature)inputquery.readObject();
				listaquery[i]=aux;
				image.addItem(aux.getName());
			}
			inputquery.close();
		}
		catch (Exception e) {System.exit(1);}		
		number = new JTextField("",3);
		p1.add(new JLabel("Immagine Query:"));
		p1.add(image);
		p1.add(new JLabel("Numero Immagini:"));
		p1.add(number);
		Border titled2 = BorderFactory.createTitledBorder(etched,"OUTPUT");
		JPanel p2 = new JPanel();
		p2.setBorder(titled2);
		output = new JTextArea(15,35);
		output.setBorder(etched);
		output.setEditable(false);
		JScrollPane scroll = new JScrollPane(output);
		p2.add(scroll);
		JPanel p3 = new JPanel();
		okButton = addButton(p3,"OK");
		settingButton = addButton(p3,"SETTING");
		clearButton = addButton(p3,"CLEAR");
		exitButton = addButton(p3,"EXIT");
		getContentPane().add(p1,"North");
		getContentPane().add(p2,"Center");
		getContentPane().add(p3,"South");
		info = new ClientConnectionInfo();
		windowInfo=new FinestraImpostazioniClient(this);
		windowInfo.showDialog(info);
	}

/* gestione degli eventi dell'interfaccia grafica */	
	public void actionPerformed(ActionEvent e)
	{
		Object source = e.getSource();
		if (source==exitButton) System.exit(0);
		else if (source==settingButton) windowInfo.showDialog(info);
		else if (source==clearButton) output.setText("");
		else if (source==okButton) 
		{ 
			output.append("Richiesta di "+number.getText()+
				" immagini simili a "+image.getSelectedItem()+'\n'+'\r');
			query();
		}
	}

/* richiesta del servizio */	
	private void query()
	{
		int k = Integer.parseInt(number.getText());
		float[] component = listaquery[image.getSelectedIndex()].getColorHistogram().toArray();
		if (! eseguiQuery(info.HostMaster1,info.PortMaster1,component,k))
			if (!eseguiQuery(info.HostMaster2,info.PortMaster2,component,k))
				output.append("IMPOSSIBILE OTTENERE IL SERVIZIO\n");
	}
	
	private boolean eseguiQuery(String host, int port, float[] component, int number)
	{
		try {
			Socket socket = new Socket(host,port);
			DataOutputStream out = new DataOutputStream( socket.getOutputStream() );
			DataInputStream in = new DataInputStream( socket.getInputStream() );
			socket.setSoTimeout(TIMEOUT);
			for (int i=0;i<64;i++) out.writeFloat(component[i]);
			out.writeInt(number);
			out.flush();
			if (in.readInt()>0)
			{
				String str;
				for (int i=0;i<number;i++)
				{
					str = in.readUTF()+'\t'+in.readUTF()+'\t'+"   "+in.readFloat();
					output.append(str+'\n');
				}
				output.append("Servizio Completato\n\n");
			}
			else output.append("Servizio non disponibile\n\n");
			in.close();
			out.close();
			socket.close();
			return true;
		}
		catch (EOFException e) 
		{ output.append("NON CI SONO ALTRE IMMAGINI !"); return true; }
		catch (Exception e)	{ return false; }
	}
	
	private JButton addButton(Container c, String name)
	{
		JButton button = new JButton(name);
		button.addActionListener(this);
		c.add(button);
		return button;
	}
	
	private JButton okButton;
	private JButton clearButton;
	private JButton settingButton;
	private JButton exitButton;
	private JTextArea output;
	private JComboBox image;
	private ImageFeature[] listaquery;
	private JTextField number;
	private JFileChooser scegli;
	private ClientConnectionInfo info;
	private FinestraImpostazioniClient windowInfo;
	private static final int TIMEOUT = 3000;
}

/* Finestra di dialogo per l'inserimento dei parametri di comunicazione */
class FinestraImpostazioniClient extends JDialog implements ActionListener
{
	public FinestraImpostazioniClient(JFrame parent)
	{
		super(parent,"SETTINGS:",true);
		Border etched = BorderFactory.createEtchedBorder();
		Border titled1 = BorderFactory.createTitledBorder(etched,"MASTER 1");
		JPanel p1=new JPanel();
		p1.setBorder(titled1);
		master1 = new JTextField("",15);
		port1 = new JTextField("",7);
		p1.add(new JLabel("HOST"));
		p1.add(master1);
		p1.add(new JLabel("   PORT"));
		p1.add(port1);
		Border titled2 = BorderFactory.createTitledBorder(etched,"MASTER 2");
		JPanel p2=new JPanel();	
		master2 = new JTextField("",15);
		port2 = new JTextField("",7);
		p2.setBorder(titled2);
		p2.add(new JLabel("HOST"));
		p2.add(master2);
		p2.add(new JLabel("   PORT"));
		p2.add(port2);
		JPanel p = new JPanel();
		p.setLayout(new GridLayout(2,1));
		p.add(p1);
		p.add(p2);
		JPanel p3 = new JPanel();
		okButton = new JButton("OK");
		okButton.addActionListener(this);
		p3.add(okButton);
		getContentPane().add(p,"Center");
		getContentPane().add(p3,"South");
		setSize(400,200);
	}

	public void actionPerformed(ActionEvent e)
	{
		Object source = e.getSource();
		if (source==okButton)
		{
			ok = true;
			setVisible(false);
		}
	}
	
	public void showDialog(ClientConnectionInfo transfer)
	{
		master1.setText(transfer.HostMaster1);
		master2.setText(transfer.HostMaster2);
		port1.setText(String.valueOf(transfer.PortMaster1));
		port2.setText(String.valueOf(transfer.PortMaster2));
		ok = false;
		show();
		if (ok)
		{
			transfer.HostMaster1=master1.getText();
			transfer.HostMaster2=master2.getText();
			transfer.PortMaster1=Integer.parseInt(port1.getText());
			transfer.PortMaster2=Integer.parseInt(port2.getText());
		}
	}
	
	private boolean ok;
	private JTextField master1;
	private JTextField master2;
	private JTextField port1;
	private JTextField port2;
	private JButton okButton;
}