package reti;

import java.text.*;
import java.awt.image.*;
import java.io.*;


/* Istogramma di colore di una immagine, ottenuto dividendo ogni componente del modello RGB in 4 range. */
public class ColorHistogram implements Serializable
{
	public ColorHistogram(BufferedImage img)

	{
		bin=new float[NUMBER_OF_BIN];
		for (int i=0;i<NUMBER_OF_BIN;i++) bin[i]=0;
		int height=img.getHeight();
		int width=img.getWidth();
		for (int y=0;y<height;y++)
			for (int x=0;x<width;x++)
			{
				int color=img.getRGB(x,y);
				int red=((color&0xff0000)/0x10000)/64;
				int green=((color&0xff00)/0x100)/64;
				int blue=(color&0xff)/64;
				bin[red*16+green*4+blue]++;
			}
		for (int i=0;i<NUMBER_OF_BIN;i++) bin[i]=bin[i]/(height*width);

	}

	ColorHistogram(float[] component)
	{
		if (component.length!=NUMBER_OF_BIN) return;
		bin = new float[NUMBER_OF_BIN];
		for (int i=0;i<NUMBER_OF_BIN;i++)
			bin[i]=component[i];
	}
	
	public float getDistance(ColorHistogram another)
	{
		float temp=0;
		for (int i=0;i<NUMBER_OF_BIN;i++)
			temp+=Math.pow(bin[i]-another.bin[i],2);
		temp=(float)(Math.sqrt(temp)/Math.sqrt(2));
		return temp;
	}

	public float[] toArray()
	{
		return bin;
	}

	public boolean equals(ColorHistogram ch)
	{
		int i = 0;
		boolean esito = true;
		while ( (i<NUMBER_OF_BIN) && (esito) )
		{
			esito = (bin[i]==ch.bin[i]);
			i++;
		}
		return esito;
	}

	public void print()
	{
		NumberFormat nf=NumberFormat.getNumberInstance();
		nf.setMinimumFractionDigits(3);
		nf.setMaximumFractionDigits(3);
		for (int i=0;i<NUMBER_OF_BIN/8;i++)
		{
			for (int x=0;x<8;x++)
				System.out.print(nf.format(bin[i*8+x])+"   ");
			System.out.println("");
		}
	}

	private float[] bin=null;
	private final int NUMBER_OF_BIN=64;
}