package reti;

import java.net.*;
import java.io.*;

public class ISGestoreRichieste extends Thread
{
 	public ISGestoreRichieste(int port, ImageFeatureCollection collection, ISCache iscache)
 	{
 		porta = port;
 		ifc = collection;	
 		cache = iscache;
 	}
 	
 	public void run()
 	{
 		try {
 			ServerSocket server = new ServerSocket(porta);
 			System.out.println("IMAGE SERVER in ascolto su IP:" + InetAddress.getLocalHost().getHostAddress() + " porta: "+porta);
 			while ( !interrupted() )
 			{
				try {
 					Socket socket = server.accept();
 					Thread t = new RisolviQuery(socket,ifc,cache);
 					t.start();
				}
				catch (IOException e) {}
 			}
 			server.close();
 		}
 		catch (Exception e) 
 		{
 			System.out.println("ECCEZIONE in ISGestoreRichieste: " + e.getMessage()); 
 			System.exit(1);
 		}
 	}
 	
 	private int porta;
 	private ImageFeatureCollection  ifc;
 	private ISCache cache;
 }
 
 class RisolviQuery extends Thread
 {
 	public RisolviQuery(Socket s, ImageFeatureCollection ifc, ISCache c)
 	{
 		socket = s;
 		collection = ifc;
 		cache = c;
 	}
 	
 	public void run()
 	{
 		try {
 			DataOutputStream out = new DataOutputStream( socket.getOutputStream() );
 			DataInputStream in = new DataInputStream( socket.getInputStream() );
 			float[] component = new float[64];
 			for (int i=0;i<64;i++) component[i] = in.readFloat();
 			int number = in.readInt();
 			if ( number>collection.getSize() ) number=collection.getSize();	
 			boolean in_cache = true;
 			ColorHistogram ch = new ColorHistogram(component);
 			QueryResult qr = cache.search(ch,number);
 			if (qr==null)
 			{
 				qr = collection.getSimilar(new ColorHistogram(component),number);
 				in_cache = false;
 			}
 			for (int i=0;i<number;i++)
 			{
 				out.writeUTF(qr.get(i).getName());
 				out.writeFloat(qr.get(i).getDistance());
 			}
 			out.flush();
 			in.close();
 			out.close();
 			socket.close();
 			if (!in_cache) cache.insert(new ISCacheItem(ch,number,qr));
 		}
 		catch (IOException e) { }
 	}
 
 	private Socket socket;
 	private ImageFeatureCollection collection;
 	private ISCache cache;
}