package reti;
import java.net.*;
import java.util.*;
import java.io.*;

/* thread per la ricezione di richieste da parte dei Client */
public class MSGestoreRichiesteClient extends Thread
{
	public MSGestoreRichiesteClient(Vector lista,int Porta,MSCoordinazioneMaster mscm)
	{
		listaIS=lista;
		porta = Porta;
		coordinazione = mscm;
	}

	public void run()
	{	
		ServerSocket server;
		try {
			server = new ServerSocket(porta);
			System.out.println("- porta per Client: "+porta);
			while ( !interrupted() )
			{
				try {
					Socket socket = server.accept();
					/* creo un nuovo thread per servire la richiesta */
					Thread t = new MSRisolviQuery(socket,listaIS,coordinazione);
					t.start();
				}
				catch (IOException e) {}
			}
			server.close();
		}
		catch(IOException e) 
		{
			System.out.println("ECCEZIONE IN Gestore Richieste Client: "+e.getMessage()); 
			System.exit(1);
		}
	}

	private int porta;
	private Vector listaIS;
	private MSCoordinazioneMaster coordinazione;
}

/* thread per il servizio di una richiesta di un Client */
class MSRisolviQuery extends Thread
{
	public MSRisolviQuery(Socket s, Vector lista, MSCoordinazioneMaster mscm)
	{
		socket = s;
		listaIS = lista;
		coordinazione = mscm;
	}
	
	public void run()
	{
		try {
			DataInputStream in = new DataInputStream(socket.getInputStream());
			DataOutputStream out = new DataOutputStream(socket.getOutputStream());
			
			/* lettura query */
			float[] component = new float[64];
			for (int i=0;i<64;i++) component[i]=in.readFloat();
			int number = in.readInt();
			MSQueryResult result;
			
			/* creo un nuovo thread per ogni Image Server registrato */
			Vector threads;
			synchronized(listaIS)
			{
				int dim=listaIS.size();
				threads = new Vector(dim);
				result = new MSQueryResult(dim*number);
				for (int i=0;i<dim;i++)
				{
					threads.add(new InterrogaIS((ISListEntry)listaIS.get(i),component,number,result,this));
					((Thread) threads.get(i)).start();
				}
			}
			
			/* attesa della risposta da tutti gli Image Server */
			for (int i=0;i<threads.size();i++) ((Thread) threads.get(i)).join();
			result.sort();	/* ordino i risultati */
			if (number>result.getSize()) number=result.getSize();
			
			/* invio risultato al Client */
			out.writeInt(number);
			for (int i=0;i<number;i++)
			{
				MSQueryResultItem item = result.get(i);
				out.writeUTF(item.getHost());
				out.writeUTF(item.getName());
				out.writeFloat(item.getDistance());
			}
			out.flush();
			in.close();
			out.close();
			socket.close();
		}
		catch (Exception e) 
		{ System.out.println("eccezione in MSRisolviQuery: "+e.getMessage()); }
		
	}	
	
	void eliminaIS(InetAddress host, int port)
	{
		ISListEntry item = new ISListEntry(host,port);
		listaIS.remove(item);
		coordinazione.notificaMessaggio(Command.DEL,host,port);
		System.out.println("EFFETTUATO UNBIND "+item);
	}
		
	
	private Socket socket;
	private Vector listaIS;
	private MSCoordinazioneMaster coordinazione;
}

/* thread per effettuare una richiesta ad un image server */
class InterrogaIS extends Thread
{
	public InterrogaIS(ISListEntry isle, float[] ch, int numero, MSQueryResult risultato, 
			MSRisolviQuery msrq)
	{
		host = isle.getIP();
		port = isle.getPort();
		component = ch;
		number = numero;
		result = risultato;
		parent = msrq;
	}

	public void run()
	{
		try {
			Socket socket = new Socket(host,port);
			DataInputStream in = new DataInputStream( socket.getInputStream() );
			DataOutputStream out = new DataOutputStream (socket.getOutputStream() );
			socket.setSoTimeout(TIMEOUT);
			
			/* invio query */
			for (int i=0;i<64;i++) out.writeFloat(component[i]);
			out.writeInt(number);
			out.flush();
			
			/* ricezione risultato */
			for (int i=0;i<number;i++)
			{
				MSQueryResultItem item = new MSQueryResultItem(host.getHostAddress(),
					in.readUTF(), in.readFloat() );
				result.add(item);
			}
			in.close();
			out.close();
			socket.close();
		}
		catch (EOFException e) 
		{}		
		catch(IOException e)
		{
			System.out.println("IMAGE SERVER SU HOST "+host.getHostAddress()+" PORT "+port+" DOWN");
			parent.eliminaIS(host,port);
		}		
	}

	private InetAddress host;
	private int port;
	private float[] component;
	private int number;
	private MSQueryResult result;
	private MSRisolviQuery parent;
	private static final int TIMEOUT=1000;
}