package reti;

import java.net.*;
import java.io.*;
import java.util.*;

/* Thread per la ricezione delle richieste dei server */
public class MSGestoreRichiesteServer extends Thread
{
	public MSGestoreRichiesteServer(int porta, Vector lista, MSCoordinazioneMaster mscm)
	{
		listaIS = lista;
		coordinazione = mscm;
		PortaInterna=porta;
	}
	
	public void run()
	{
		try {
			ServerSocket server=new ServerSocket(PortaInterna);
			System.out.println("- porta per Server: "+PortaInterna);
			while ( !interrupted() )
			{
				try {
					Socket income=server.accept();
					/* creo un nuovo thread per servire la richiesta */
					Thread t=new ThreadedGestoreServer(listaIS,income,coordinazione);
					t.start();
				}
				catch (IOException e) {}
			}
		}
		catch (Exception e) 
		{
			System.out.println("Eccezione in Gestore Richieste Server: "+e.getMessage()); 
			System.exit(1);
		}
	}

	private Vector listaIS;
	private int PortaInterna;
	private MSCoordinazioneMaster coordinazione;
}

/* thread per servire la richiesta di un server */
class ThreadedGestoreServer extends Thread
{
	public ThreadedGestoreServer(Vector isl, Socket s, MSCoordinazioneMaster mscm)
	{
		listaIS = isl;
		socket = s;
		coordinazione = mscm;
	}
	
	public void run()
	{
		try {
			ObjectInputStream in = new ObjectInputStream(socket.getInputStream());
			ObjectOutputStream out = new ObjectOutputStream(socket.getOutputStream());
			socket.setTcpNoDelay(true);
			int type = in.readInt();
			ISListEntry isle;
			switch (type)
			{
				case Command.BIND:		/* richiesta di BIND */
					isle = new ISListEntry(socket.getInetAddress(),in.readInt());
					synchronized(listaIS)
						{ if ( listaIS.indexOf(isle)==-1 ) listaIS.add(isle); }
					coordinazione.notificaMessaggio(Command.INS,isle.getIP(),isle.getPort());
					out.writeInt(Command.OK);
					out.flush();
					System.out.println("EFFETTUATO BIND  "+isle);
					break;
				case Command.UNBIND:	/* richiesta di UNBIND */
					isle = new ISListEntry(socket.getInetAddress(),in.readInt());
					listaIS.remove(isle); 
					coordinazione.notificaMessaggio(Command.DEL,isle.getIP(),isle.getPort());
					out.writeInt(Command.OK); out.flush();
					System.out.println("EFFETTUATO UNBIND  "+isle);
					break;
				case Command.INS:		/* richiesta di INS */
					isle = new ISListEntry((InetAddress) in.readObject(),in.readInt());
					synchronized(listaIS) 
						{ if (listaIS.indexOf(isle)==-1) listaIS.add(isle); }
					out.writeInt(Command.OK);
					out.flush();
					System.out.println("EFFETTUATO BIND  "+isle);
					break;
				case Command.DEL:		/* richiesta di DEL */
					isle = new ISListEntry( (InetAddress) in.readObject(),in.readInt());
					listaIS.remove(isle);
					out.writeInt(Command.OK);
					out.flush();
					System.out.println("EFFETTUATO UNBIND  "+isle);
					break;
				case Command.UPDATE:	/* richiesta di UPDATE */
					out.writeObject(listaIS);
					break;	
			}
			in.close();
			out.close();
			socket.close();
		}
		catch (Exception e) 
		{
			System.out.println("ECCEZIONE DURANTE IL SERVIZIO DI UNA RICHIESTA SERVER: "
			+e.getMessage() );
		}
	}

	private Vector listaIS;		/* elenco Image Server */
	private Socket socket;
	private MSCoordinazioneMaster coordinazione;
}